/**********************************************************************************
         
	 Aug 2012 : gw@westdale.demon.co.uk --  West Dale Design -- Gareth Williams
	 
	 Based on an idea by Shanfan Huang http://shanfanhuang.com
	 
	 Recoded to create pattern as a group and with several more options for scaling, flipping
	 Also coded in a way to make it easier to add more patterns on a rectangular grid.
	 To add a pattern, create a unit which has a vertical drop and indicate the horizontal/vertical steps.
	 It forms a unit, then a row group then the group as a whole.
	 
	 Variator and Themed Colours recommended for filling 
	 or use the Raster facility ( select the pattern group and a raster and click ) 
	 
	 Version 'alt' allows alternate drawing for some shapes   Sept 2012
	 and adds a Captured jigsaw shape as pattern Capture1 
	 ( for which I used the getPathScript after drawing them normally ! )

        Version alt2 allows specification of size in inches and mm   Apr 2015
                    
***********************************************************************************/

// data for interface

// shape values 
var triEq = 'triangle-eq';
var triIso = 'triangle-iso';
var triLeft = 'diagonal';
var tri2 = 'triangle2';
var rect = 'rectangle';
var rect2 = 'rectangle2';
var diamond = 'diamond';
var diamond2 = 'diamond2';
var hexP = 'packed hex';
var hexL = 'loose hex';
var brick_s = 'stretcher';
var brick_e = 'english';
var brick_f = 'flemish';
var circ1 = 'circles1';
var circ2 = 'circles2';
var trap1 = 'trapez1';
var trap2 = 'trapez2';
var shook = 'S hook';
var capture1 = 'capture1';

// scaling values to work in mm instead of points  Apr 2015
var point = 'point';
var mm = 'mm';
var inch = 'inch';
var pointScale = 1; //default
var pointsPerInch = 72;
var pointsPerMm = 72 * 0.03937;


// fit values 
 var rowcol = 'rowcol';
 var sel = 'selection';
 var page = 'page';
 
 // flip values 
 var flipOff = 'off';
 var flip = 'flip';
 var flipAlt = 'alternate';
 var flipAlt2 = 'alternate2';
 var allowFlipShear = false;
 
// default values 
var values = {
    width: 40,
    height: 40,
    symm: true,
    rows: 15,
    columns: 10,
    area:rowcol,
    shape: triLeft,
    objFlip: flipOff,
    objShear: 0,
    rowFlip: flipOff,
    rowStagger:false,
    alts: false,
    scale: point
};

// palette components 
var components = {
    area: {
       type: 'list',
       label : 'fill area',
       options : [rowcol, sel, page ]
    },
    shape: {
       type: 'list',
       label: 'shape',
       options: [triEq, triIso,tri2, triLeft, 
                 trap1, trap2, 
                 hexP, hexL, 
                 circ1, circ2, 
                 rect, rect2, 
                 diamond, diamond2, 
                 shook, 
                 brick_s, brick_e, brick_f, 
                 capture1],
       onChange: function(value){ 
                     // some shapes do not work with different x and y heights
                     if (value == triEq) { 
                         components.symm.enabled = false; // equi triangles always same shape
                         components.symm.value = true ;
                         values.height=values.width;
                         components.height.enabled = false
                     } else if (value == brick_e ||value == brick_e ||value == brick_e) {
                        components.symm.enabled = false; // bricks ignore y-size
                     } else {
                         components.symm.enabled = true;
                         components.height.enabled = true
                     }  
                     // only flip asymmetric objects 
                     if (value == triLeft || value == rect2) {  //  asymmetric - rect 2 is asymmetric if 'alt'
                         components.objFlip.enabled = true;
                     } else {
                         values.objFlip = flipOff; 
                         components.objFlip.enabled = false; 
                     }  
                     // some shapes wont alternate nicely so disable this to show the user
                     if (value == triEq || value == triIso || value == triLeft || value == rect2 
                         || value == diamond  || value == diamond2 || value == hexP || value == trap1 || value ==  trap2 ) {
                        components.alts.enabled = true;
                     } else {
                        components.alts.enabled = false;
                     }  
                     // dont flip rect2 unless alt is true
                     if (value == rect2) {
                        components.objFlip.enabled  = components.alts.enabled
                     }
                 }
    },
    alts: {
        type: 'boolean',
        label: 'alts'
   },
    symm: {
        type: 'boolean',
        label: 'w=h',
        onChange: function(value) { 
                     if(value){
                         values.height = values.width;
                         components.height.enabled = false
                     }else{
                          components.height.enabled = true
                     } 
                  }
   },
    width: {
        type: 'number',
        label: 'width',
        onChange: function(value){ 
                      if (!components.height.enabled) { values.height = value}
                  }
    },
    height: {
        type: 'number',
        label: 'height',
        enabled: false
    },
scale: {  // width & height in points, inches or  millimeters  apr 2015
	   type: 'list',
	   label: 'units',
	   options: [point, mm, inch],
	   onChange: function(value) {
	      if (value==mm) {
		 pointScale = pointsPerMm
	      } else if (value == inch) {
		 pointScale = pointsPerInch
	      } else {
		 pointScale = 1 
	      }
	   }
     },
    rows: {
        type: 'number',
        label: 'Rows',
        steppers: true,
        min: 1,
        increment: 1
    },
    columns: {
        type: 'number',
        label: 'Columns',
        steppers: true,
        min: 1,
        increment: 1
    },
    objFlip: {
        type: 'list',
        label: 'obj flip',
       options: [flipOff, flip, flipAlt, flipAlt2],
       enabled: true,
        onChange: function(value) { 
                     if(allowFlipShear) {return}
                     if(value!= flipOff){
                         values.objShear = 0;  // shearing does not play nicely
                         components.objShear.enabled = false
                     }else{
                          components.objShear.enabled = true
                     } 
                  }
    },
    objShear: {
       type: 'number',
       label: 'shear%',
       enabled: true
    },
    rowFlip: {
        type: 'list',
        label: 'row flip',
       options: [flipOff, flip, flipAlt, flipAlt2],
       enabled: true
    },
    rowStagger:{
        type: 'boolean',
        label: 'stagger'
    },
    run: {
        type: 'button',
        value: 'Make',
        onClick: function() {run();} 
    },
    raster: {
        type: 'button',
        value: 'Raster',
        onClick: function() {rasterColour();} 
    }
};
    
var palette = new Palette('Grid Generator', components, values);



// --------------------------------------------------------------
  
function run() {

	var group = new Group(); // the end result
	var row   = new Group(); // the row
	
	// passed to shape makers....
	var userSize;
	var targetType; 
	var targetSize; // target size for row cols
	var targetOrigin; 
	// returned by shape makers ....
	var unit;     // either path item or a group 	 
	var stepSize; // horiz / vert step size for unit
	var rowsPerUnit; // a shape can have the appearance of more than one row - create fewer rows than user specifies

 	// brick sizes - approx metric with mortar allowance
 	var br1 = 217.5;
 	var br2 = br1/2;
 	var br3 = br1/3;


	function defineTarget() { // ----------------------------------

	    targetType = values.area;
	    userSize = new Point(Math.round(values.width* pointScale),Math.round(values.height * pointScale)) ;  // pointScale Apr 2015
	    shear = values.objShear;
	    
	    if (targetType == page) {
		  targetSize = new Point(document.bounds.width, document.bounds.height);
		  targetOrigin = document.bounds.topLeft;

	    } else if (targetType == sel) {
			var objList = document.getItems({selected: true});
			if (! objList.length) {
				Dialog.alert('Select Object for \'sel\' option !');
				return false;
			}else{
				var objBounds = objList[0].bounds;
				targetSize = new Point(objBounds.width, objBounds.height);
				targetOrigin = objBounds.topLeft;
			}

	   } else { // row col
			targetSize = new Point(values.columns, values.rows);
			targetOrigin = document.bounds.topLeft;
	   }

        }// fn defineTarget
    
    
	function defineUnit () {  // ----------------------------------
	
		switch (values.shape) {
		  // draw unit at 0,0 and set step size etc 
		  case triEq :
			 defineEquiTri(); break;  
		  case triIso :
			  defineIsoTri(); break;  
		  case triLeft :
			  defineDiagonal(); break;
		  case tri2 :
			  defineTriangle2(); break;
		  case hexP :
			  definePackedHex(); break;
		  case hexL :
			  defineLooseHex(); break;
		  case diamond :
			  defineDiamond(); break;
		  case diamond2 :
			  defineDiamond2(); break;
		  case rect :
			  defineRect(); break;
		  case rect2 :
			  defineRect2(); break;
	      case brick_s:
	          defineStretcherBond(); break;
	      case brick_e:
	          defineEnglishBond(); break;
	      case brick_f:
	          defineFlemishBond(); break;
	      case circ1:
	          defineCircles1(); break;
	      case circ2:
	          defineCircles2(); break;
	      case trap1:
	          defineTrap1(); break;
	      case trap2:
	          defineTrap2(); break;
	      case shook:
	          defineShook(); break;
	      case capture1:
	          defineCapture1(); break;
		}
		

	}//defineUnit
	
	
	// --- define the shapes -----------------------------------------------------------------
	
	function defineEquiTri() {
		// the unit is two equilateral triangles -- ignores the height and symmetry indicators
		// just makes it easier for the user - else is same as isosceles triangle! 
		// 'alts' suppresses one triangle
		unit = new Group();
		var tri1 = new Path.RegularPolygon(document.bounds.topLeft, 3, 10);
		unit.appendTop( tri1);
		if (!values.alts) {
			var tri2 = tri1.clone();
			tri2.scale(1,-1);
			tri2.translate(new Point(tri2.bounds.width/2,0)); 
			unit.appendTop( tri2);
	    }
		var scale = userSize.x/tri1.bounds.width
		unit.scale( scale, scale );  // scale to specified size - uniform (ignore y scaling)
		var scaledStep = new Point(tri1.bounds.size); // before shearing
		unit.shear(shear/100,0);  
		stepSize = scaledStep ; 
		rowsPerUnit = 1;
	}
	
	function defineTriangle2() {
		// the unit is 2 large and 2 small triangles -- no alts
		unit = new Group();
		var tri1 = new Path.RegularPolygon(document.bounds.topLeft, 3, 10);
		var tri2 = tri1.clone();
		tri2.scale(0.5,0.5);
		tri2.translate(new Point(tri1.bounds.width*0.5,-tri1.bounds.height*0.25)); 
		unit.appendTop( tri1);
		unit.appendTop( tri2);
		var tri3 = tri1.clone();
		var tri4 = tri2.clone();
		tri3.translate(new Point(tri3.bounds.width*0.5,tri3.bounds.height));
		tri4.translate(new Point(tri3.bounds.width*0.5,tri3.bounds.height));
		unit.appendTop( tri3);
		unit.appendTop( tri4);
		var scalex = userSize.x/tri1.bounds.width
		var scaley = userSize.y/tri1.bounds.height
		unit.scale(scalex, scaley);  // scale to specified size - uniform (ignore y scaling)
		var scaledStep = new Point(tri1.bounds.width,2*tri1.bounds.height); // before shearing
		unit.shear(shear/100,0);  
		stepSize = scaledStep ; 
		rowsPerUnit = 2;
	}
	
	function defineIsoTri() {
		// the unit is two triangles
		unit = new Group();
		var tri1 = new Path.RegularPolygon(document.bounds.topLeft, 3, 10);
		unit.appendTop(tri1);		
		if (!values.alts) {
			var tri2 = tri1.clone();
			tri2.scale(1,-1);
			tri2.translate(new Point(tri2.bounds.width/2,0)); 
			unit.appendTop(tri2);
		}
		unit.scale( userSize.x/tri1.bounds.width, userSize.y/tri1.bounds.height ); 
		var scaledStep = new Point(tri1.bounds.size); // before shearing
		unit.shear(shear/100,0);
		stepSize = scaledStep ; 
		rowsPerUnit = 1;
	}
	
	function defineTrap1() {
		// the unit is two trapezia
		unit = new Group();
		var trap1 = new Path();
		var w = 10; var h = 4;
		trap1.add( new Point(0,0), new Point(h/2,h), new Point(w-h/2, h),new Point(w,0));
		trap1.closed=true;
		trap1.position = document.bounds.topLeft;
		unit.appendTop(trap1);
		if (!values.alts) {
			var trap2 = trap1.clone();
			trap2.scale(1,-1);
			trap2.translate(new Point(w-h/2,0)); 
			unit.appendTop(trap2);
	    }
		var step = new Point(2*w-h, h);  // before scaling
		var scale = new Point(userSize.x/w, userSize.y/h);
		unit.scale(scale.x, scale.y); 
		var scaledStep = new Point(step.x*scale.x, step.y*scale.y); // before shearing
		unit.shear(shear/100,0);
		stepSize = scaledStep ; 
		rowsPerUnit = 1;
	}
	
	function defineTrap2() {
		// the unit is two trapezia
		unit = new Group();
		var trap1 = new Path();
		var w = 10; var h = 4;
		trap1.add( new Point(0,0), new Point(h,h), new Point(w-h, h),new Point(w,0));
		trap1.closed=true;
		trap1.position = document.bounds.topLeft
		unit.appendTop(trap1);	
		if (!values.alts) {
			var trap2 = trap1.clone();
			trap2.scale(1,-1);
			trap2.translate(new Point(w-h,0)); 
			unit.appendTop(trap2);		
		}
		var step = new Point(2*w-2*h, h);  // before scaling
		var scale = new Point(userSize.x/w, userSize.y/h);
		unit.scale(scale.x, scale.y); 
		var scaledStep = new Point(step.x*scale.x, step.y*scale.y); // before shearing
		unit.shear(shear/100,0);
		stepSize = scaledStep ; 
		rowsPerUnit = 1;
	}
	
	function defineDiagonal(){
		// triLeft -  the unit is two triangles
		unit = new Group();
		var tri1 = new Path();
		tri1.add( new Point(0,0), new Point(userSize), new Point(0,userSize.y));
		tri1.closed = true;
		unit.appendTop(tri1);
		if (!values.alts) {
			var tri2 = tri1.clone();
			tri2.scale(-1,-1);	
			unit.appendTop(tri2);
		}
		unit.shear(shear/100,0);
		stepSize = userSize ;
		rowsPerUnit = 1; 
	}
	
	
	function defineDiamond2(){
	    // four triangles as a unit
		unit = new Group();
	    var dia11 = new Path();
	    var dia12 = new Path();
		dia11.add(new Point(0, -5),new Point(-5, 0),new Point(0,5));
		dia11.closed = true	
		dia12.add(new Point(0,-5),new Point(5, 0),new Point(0,5));
		dia12.closed = true;
		unit.appendTop(dia11);
		unit.appendTop(dia12);		
		if (!values.alts) {
			var dia21 = dia11.clone();
			dia21.translate(new Point(5,5))
			var dia22 = dia12.clone();	
			dia22.translate(new Point(5,5))
			unit.appendTop(dia21);
			unit.appendTop(dia22);
		}
		unit.scale( userSize.x/10, userSize.y/10 ); 
		unit.shear(shear/100,0);  
		stepSize = userSize ; 
		rowsPerUnit = 1;
	}
	
	
	function defineDiamond(){
	// two diamonds as a unit
		unit = new Group();
		var dia1 = new Path();
		dia1.add(new Point(0, 10),new Point(10, 0),new Point(0, -10),new Point(-10, 0));
		dia1.closed = true
		unit.appendTop( dia1);		
		if (!values.alts) {
			var dia2 = dia1.clone();
			dia2.position = new Point(10,10);
			unit.appendTop( dia2);
		}
		unit.scale( userSize.x/20, userSize.y/20 ); 
		unit.shear(shear/100,0);  
		stepSize = userSize ; 
		rowsPerUnit = 1;
	}
	
	
	function defineRect(){
		// the unit is one rectangle
		
		var w = userSize.x;
		var h = userSize.y;
		var p1 = new Path();
		p1.add(new Point(0,0),new Point(w,0),new Point(w,h),new Point(0,h));
		p1.closed=true;
		unit = p1;
		unit.shear(shear/100,0);
		stepSize = userSize ; 
		rowsPerUnit = 1;
	}
	
	function defineRect2(){
		// the unit is tow items that look like 4 rectangles 
		var w = userSize.x;
		var h = userSize.y;
		unit = new Group();
		var p1 = new Path();
		p1.add(new Point(0,0),new Point(w/2,0),new Point(w/2,h),new Point(w,h),new Point(w, h/2),new Point(0,h/2));  // fix Apr 2015
		p1.closed=true;
		unit.appendTop(p1);	
		if (!values.alts) {
			var p2 = p1.clone();
			p2.scale(-1,1);
			unit.appendTop(p2);        
		}
		unit.shear(shear/100,0);
		stepSize = userSize ; 
		rowsPerUnit = 2;
	}
	
	function defineRect2a(){
		// the unit is an item that look like 2 rectangles - mirror of the alt version of rect2
		var w = userSize.x;
		var h = userSize.y;
		unit = new Path();
		unit.add(new Point(0,h),new Point(w/2,h),new Point(w/2,0),new Point(w,0),new Point(w, h/2),new Point(0,h/2));
		unit.closed=true;
		unit.shear(shear/100,0);
		stepSize = userSize ; 
		rowsPerUnit = 2;
	}
	
	
	function defineShook(){
		// the unit is one S hook shape
		unit = new Path();
		var thick = userSize.y/3; 
		var len = userSize.x;
		var height = userSize.y;
		if (thick > len/5) {thick = len/5 } // else overlap
		unit.add(new Point(0, thick), new Point(thick, thick), new Point(thick, height-thick), new Point(2*thick, height-thick),
		         new Point(2*thick, 0), new Point(len, 0), new Point(len, height-thick), new Point(len-thick, height-thick),
		         new Point(len-thick, thick), new Point(len-2*thick, thick), new Point(len-2*thick, height), new Point(0, height));
		unit.closed = true;
		unit.shear(shear/100,0);
		stepSize = new Point(len-2*thick,height);
		rowsPerUnit = 1;
	}
	
	
	function definePackedHex(){
	    // two hexes form a unit
		unit = new Group();
		var radius = 10; // initial radius before user scaling
		var hex1 = new Path.RegularPolygon(document.bounds.topLeft, 6, radius);
		unit.appendTop( hex1);	
		if (!values.alts) {
		var hex2 = hex1.clone();
		hex2.position = new Point(hex1.bounds.width - radius/2, hex1.bounds.height/2);
		unit.appendTop( hex2);
		}
		var steps = new Point(hex1.bounds.width + radius, hex1.bounds.height); //unscaled
		var scalex = userSize.x/hex1.bounds.width;
		var scaley = userSize.y/hex1.bounds.height;
		steps.x = steps.x * scalex;
		steps.y = steps.y * scaley; 
		unit.scale(scalex, scaley); 
		unit.shear(shear/100,0);  
		stepSize = steps ; 
		rowsPerUnit = 1;
	}
	
	
	function defineLooseHex(){
	    // one hex is the unit
		unit = new Path.RegularPolygon(document.bounds.topLeft, 6, 10);
		unit.scale( userSize.x/unit.bounds.width, userSize.y/unit.bounds.height );
		unit.shear(shear/100,0);
		stepSize = userSize ; 
		rowsPerUnit = 1;
	}
		
	function defineStretcherBond() {  // size is longest side of brick; ignore y scaling
	// 2bricks in two rows is a unit
		unit = new Group();
		var brick1 = new Path.Rectangle(document.bounds.topLeft, new Size(br1, br3))
		var brick2 = brick1.clone();
		brick2.translate(new Point(br1/2, br3));
		unit.appendTop(brick1);
		unit.appendTop(brick2);
		unit.scale(userSize.x/brick1.bounds.width, userSize.x/brick1.bounds.width);
		unit.shear(shear/100,0);  
		stepSize = new Point(brick1.bounds.width, brick1.bounds.height*2);
		rowsPerUnit = 2;
	}
	
	function defineEnglishBond() {
	  //3bricks in two rows is a unit
		unit = new Group();
		var TL = document.bounds.topLeft;
		var brick1 = new Path.Rectangle(TL, new Size(br1, br3));
		var brick2 = new Path.Rectangle(TL+new Point(br2*0.5,br3), new Size(br2,br3));
		var brick3 = new Path.Rectangle(TL+new Point(br2*1.5,br3), new Size(br2,br3));
		unit.appendTop(brick1);
		unit.appendTop(brick2);
		unit.appendTop(brick3);
		unit.scale(userSize.x/brick1.bounds.width, userSize.x/brick1.bounds.width);
		unit.shear(shear/100,0);  
		stepSize = new Point(brick1.bounds.width, brick1.bounds.height*2);
		rowsPerUnit = 2;
	
	}
	
	function defineFlemishBond() {
	//  4bricks in two rows is a unit
		unit = new Group();
		var TL = document.bounds.topLeft;
		var brick1 = new Path.Rectangle(TL, new Size(br1,br3));
		var brick2 = new Path.Rectangle(TL+new Point(br1,0), new Size(br2,br3));
		var brick3 = new Path.Rectangle(TL+new Point(br2*0.5,br3), new Size(br2,br3));
		var brick4 = new Path.Rectangle(TL+new Point(br2*1.5,br3), new Size(br1,br3));
		unit.appendTop(brick1);
		unit.appendTop(brick2);
		unit.appendTop(brick3);
		unit.appendTop(brick4);
		unit.scale(userSize.x/brick1.bounds.width, userSize.x/brick1.bounds.width);
		unit.shear(shear/100,0);  
		stepSize = new Point(brick1.bounds.width+brick2.bounds.width, brick1.bounds.height*2);
		rowsPerUnit = 2;
}


	function defineCircles1() {
	//  Circle+Circle is a unit
	    var rad = 10;  // but user size is taken as diameter 
		unit = new Group();
		var TL = document.bounds.topLeft;
		var c1 = new Path.Circle(TL, rad);
		var c2 = new Path.Circle(TL+new Point(rad, Math.sqrt(3*rad*rad)), rad);
		unit.appendTop(c1);
		unit.appendTop(c2);
		var steps = new Point(userSize.x, Math.sqrt(3*rad*rad)*userSize.y/rad);
		unit.scale(0.5*userSize.x/rad, 0.5*userSize.y/rad);
		unit.shear(shear/100,0);  
		stepSize = steps;
		rowsPerUnit = 2;
}

	function defineCircles2() {
	//  Circle+smallCircle is a unit
		unit = new Group();
		var TL = document.bounds.topLeft;
		var c1 = new Path.Circle(TL, 10);
		var c2 = new Path.Circle(TL+new Point(10,10), (Math.sqrt(2)-1)*10);
		unit.appendTop(c1);
		unit.appendTop(c2);
		unit.scale(userSize.x/c1.bounds.width, userSize.y/c1.bounds.height);
		unit.shear(shear/100,0);  
		stepSize = userSize;
		rowsPerUnit = 1;
}


	function defineCapture1() {
		// the unit is a group inserted from the capture script
		unit = new Group();
		// ----------------
		
		// 4 jigsaw pieces captured by getPath script
		var obj0 = new Path();
		obj0.add(new Segment(266.041015625,368.51708984375,-6.36328125,13.26953125,6.36376953125,-13.267578125));
		obj0.add(new Segment(305.99755859375,371.99267578125,-18.1787109375,-2.8564453125,18.17822265625,2.8564453125));
		obj0.add(new Segment(335.13037109375,344.16748046875,0.2294921875,29.150390625,-0.2294921875,-29.1513671875));
		obj0.add(new Segment(307.716796875,308.02490234375,12.83349609375,-6.3876953125,-12.83447265625,6.388671875));
		obj0.add(new Segment(272.40478515625,319.14306640625,12.58251953125,7.2197265625,-12.58251953125,-7.22265625));
		obj0.add(new Segment(269.30322265625,259.6708984375,0,0,0,0));
		obj0.add(new Segment(301.16845703125,258.93603515625,-22.31640625,-0.6728515625,22.31640625,0.6728515625));
		obj0.add(new Segment(341.7080078125,245.79248046875,4.34619140625,9.728515625,-11.26708984375,-25.216796875));
		obj0.add(new Segment(368.7021484375,205.3388671875,-20.02880859375,0.05810546875,17.39111328125,-0.05029296875));
		obj0.add(new Segment(404.35400390625,232.40673828125,0,-21.85400390625,0,17.8056640625));
		obj0.add(new Segment(400.87548828125,259.60888671875,-26.521484375,0,11.3046875,0));
		obj0.add(new Segment(466.42236328125,258.2021484375,0,0,0,0));
		obj0.add(new Segment(466.10205078125,323.25048828125,10.6875,-6.376953125,-10.201171875,6.0859375));
		obj0.add(new Segment(437.47314453125,313.87939453125,10.404296875,5.384765625,-10.404296875,-5.384765625));
		obj0.add(new Segment(415.24755859375,344.34423828125,0.1865234375,-24.572265625,-0.1865234375,24.572265625));
		obj0.add(new Segment(438.86669921875,367.79931640625,-14.7373046875,2.408203125,14.7373046875,-2.408203125));
		obj0.add(new Segment(471.26123046875,364.86962890625,-5.1591796875,-11.18359375,5.1591796875,11.185546875));
		obj0.add(new Segment(464.19384765625,435.857421875,0,0,0,0));
		obj0.add(new Segment(374.5546875,435.203125,1.2861328125,23.95703125,-0.87109375,-16.2265625));
		obj0.add(new Segment(385.0009765625,404.7861328125,0,25.3515625,0,-12.173828125));
		obj0.add(new Segment(351.88427734375,386.3603515625,8.97900390625,-1.86767578125,-10.7080078125,2.2275390625));
		obj0.add(new Segment(322.4130859375,416.5146484375,-0.869140625,-17.390625,0.4677734375,9.349609375));
		obj0.add(new Segment(321.93408203125,442.67578125,23.53515625,-1.634765625,-20.16357421875,1.400390625));
		obj0.add(new Segment(266.86376953125,437.4462890625,0,0,0,0));
		obj0.closed = true;
		var obj1 = new Path();
		obj1.add(new Segment(464.19384765625,50.1513671875,0,0,0,0));
		obj1.add(new Segment(464.06689453125,129.9150390625,-17.298828125,-10.35107421875,17.2978515625,10.35107421875));
		obj1.add(new Segment(496.87255859375,121.87548828125,-10.5439453125,4.54248046875,12.7958984375,-5.51220703125));
		obj1.add(new Segment(537.359375,153.40673828125,-1.28125,-12.76513671875,1.24609375,12.4228515625));
		obj1.add(new Segment(507.92919921875,193.7548828125,19.98876953125,0.689453125,-25.216796875,-0.86962890625));
		obj1.add(new Segment(465.94580078125,175.916015625,9.798828125,-10.43701171875,-9.798828125,10.43701171875));
		obj1.add(new Segment(466.42236328125,258.2021484375,0,0,0,0));
		obj1.add(new Segment(400.87548828125,259.60888671875,11.3046875,0,-26.521484375,0));
		obj1.add(new Segment(404.35400390625,232.40673828125,0,17.8056640625,0,-21.85400390625));
		obj1.add(new Segment(368.7021484375,205.3388671875,17.39111328125,-0.05029296875,-20.02880859375,0.05810546875));
		obj1.add(new Segment(341.7080078125,245.79248046875,-11.26708984375,-25.216796875,4.34619140625,9.728515625));
		obj1.add(new Segment(301.16845703125,258.93603515625,22.31640625,0.6728515625,-22.31640625,-0.6728515625));
		obj1.add(new Segment(269.30322265625,259.6708984375,0,0,0,0));
		obj1.add(new Segment(276.6298828125,178.1787109375,5.1591796875,13.27001953125,-5.1591796875,-13.267578125));
		obj1.add(new Segment(244.2353515625,181.654296875,14.73828125,-2.85693359375,-14.73779296875,2.85693359375));
		obj1.add(new Segment(220.6162109375,153.82861328125,-0.18603515625,29.1513671875,0.18603515625,-29.15087890625));
		obj1.add(new Segment(242.841796875,117.68701171875,-10.40478515625,-6.38818359375,10.4052734375,6.38818359375));
		obj1.add(new Segment(271.470703125,128.8046875,-10.201171875,7.2197265625,10.201171875,-7.22265625));
		obj1.add(new Segment(266.85498046875,51.90185546875,0,0,0,0));
		obj1.add(new Segment(323.87744140625,57.48681640625,-20.05419921875,2.52001953125,20.0556640625,-2.52001953125));
		obj1.add(new Segment(322.4130859375,31.01025390625,0.46728515625,9.349609375,-0.86962890625,-17.39111328125));
		obj1.add(new Segment(353.71728515625,0.57568359375,-10.85400390625,1.154296875,9.1201171875,-0.97021484375));
		obj1.add(new Segment(385.021484375,19.7060546875,0,-12.173828125,0,25.3515625));
		obj1.add(new Segment(374.69873046875,50.0888671875,-0.87109375,-16.22705078125,1.28564453125,23.9580078125));
		obj1.add(new Segment(463.51611328125,50.0888671875,0,0,0,0));
		obj1.closed = true;
		var obj2 = new Path();
		obj2.add(new Segment(266.85498046875,51.90185546875,0,0,0,0));
		obj2.add(new Segment(174.56494140625,54.5927734375,25.576171875,-9.8310546875,-11.35009765625,4.36279296875));
		obj2.add(new Segment(181.62939453125,76.6025390625,-5.9208984375,-5.79296875,5.921875,5.79296875));
		obj2.add(new Segment(160.12646484375,109.0029296875,36.4921875,-0.001953125,-43.865234375,0.00244140625));
		obj2.add(new Segment(140.013671875,66.24853515625,-6.04345703125,15.80517578125,6.04296875,-15.80517578125));
		obj2.add(new Segment(123.73828125,51.2666015625,0,0,0,0));
		obj2.add(new Segment(74.08154296875,50.0888671875,0,0,0,0));
		obj2.add(new Segment(74.2021484375,130.64892578125,-17.2978515625,-10.35107421875,17.2978515625,10.35107421875));
		obj2.add(new Segment(107.0087890625,122.609375,-10.54443359375,4.54248046875,12.79541015625,-5.51220703125));
		obj2.add(new Segment(147.49462890625,154.140625,-1.28076171875,-12.76513671875,1.24658203125,12.4228515625));
		obj2.add(new Segment(118.06494140625,194.48876953125,19.98828125,0.689453125,-25.21728515625,-0.86962890625));
		obj2.add(new Segment(76.08154296875,176.64990234375,9.79931640625,-10.43701171875,-9.79931640625,10.43701171875));
		obj2.add(new Segment(76.55810546875,258.93603515625,0,0,0,0));
		obj2.add(new Segment(107.05322265625,260.25341796875,-22.31640625,0.64453125,22.31640625,-0.64453125));
		obj2.add(new Segment(153.71728515625,267.5322265625,4.34619140625,-9.3193359375,-11.26708984375,24.15869140625));
		obj2.add(new Segment(174.5869140625,311.60009765625,-20.02880859375,0,24.34765625,0));
		obj2.add(new Segment(210.23876953125,285.66845703125,0,20.9365234375,0,-17.05712890625));
		obj2.add(new Segment(206.7607421875,259.60888671875,-36.29248046875,-0.6728515625,11.30224609375,0.20947265625));
		obj2.add(new Segment(269.30322265625,259.6708984375,0,0,0,0));
		obj2.add(new Segment(276.6298828125,178.1787109375,5.1591796875,13.27001953125,-5.1591796875,-13.267578125));
		obj2.add(new Segment(244.2353515625,181.654296875,14.73828125,-2.85693359375,-14.73779296875,2.85693359375));
		obj2.add(new Segment(220.6162109375,153.82861328125,-0.18603515625,29.1513671875,0.18603515625,-29.15087890625));
		obj2.add(new Segment(242.841796875,117.68701171875,-10.40478515625,-6.38818359375,10.4052734375,6.38818359375));
		obj2.add(new Segment(271.470703125,128.8046875,-10.201171875,7.2197265625,10.201171875,-7.22265625));
		obj2.closed = true;
		var obj3 = new Path();
		obj3.add(new Segment(266.30615234375,368.51708984375,-6.36328125,13.26953125,6.36376953125,-13.267578125));
		obj3.add(new Segment(306.2626953125,371.99267578125,-18.1787109375,-2.8564453125,18.17822265625,2.8564453125));
		obj3.add(new Segment(335.3955078125,344.16748046875,0.2294921875,29.150390625,-0.2294921875,-29.1513671875));
		obj3.add(new Segment(307.98193359375,308.02490234375,12.83349609375,-6.3876953125,-12.83447265625,6.388671875));
		obj3.add(new Segment(272.669921875,319.14306640625,12.58251953125,7.2197265625,-12.58251953125,-7.22265625));
		obj3.add(new Segment(269.568359375,259.6708984375,0,0,0,0));
		obj3.add(new Segment(207.02587890625,259.60888671875,11.30224609375,0.20947265625,-36.29248046875,-0.6728515625));
		obj3.add(new Segment(210.50390625,285.66845703125,0,-17.05712890625,0,20.9365234375));
		obj3.add(new Segment(174.85205078125,311.60009765625,24.34765625,0,-20.02880859375,0));
		obj3.add(new Segment(153.982421875,267.5322265625,-11.26708984375,24.15869140625,4.34619140625,-9.3193359375));
		obj3.add(new Segment(107.318359375,260.25341796875,22.31640625,-0.64453125,-22.31640625,0.64453125));
		obj3.add(new Segment(76.8232421875,258.93603515625,0,0,0,0));
		obj3.add(new Segment(76.3466796875,323.60888671875,10.201171875,-6.087890625,-10.201171875,6.0859375));
		obj3.add(new Segment(47.7177734375,314.23779296875,10.40478515625,5.384765625,-10.40478515625,-5.384765625));
		obj3.add(new Segment(25.4921875,344.70263671875,0.18603515625,-24.572265625,-0.18603515625,24.572265625));
		obj3.add(new Segment(49.111328125,368.15771484375,-14.73779296875,2.408203125,14.73779296875,-2.408203125));
		obj3.add(new Segment(81.505859375,365.22802734375,-5.1591796875,-11.18359375,5.1591796875,11.185546875));
		obj3.add(new Segment(74.46728515625,435.857421875,0,0,0,0));
		obj3.add(new Segment(122.06005859375,435.455078125,0,0,0,0));
		obj3.add(new Segment(140.33544921875,451.4375,5.66455078125,-18.193359375,-5.0302734375,16.15625));
		obj3.add(new Segment(159.5,492.5,-43.865234375,0.001953125,36.4921875,-0.001953125));
		obj3.add(new Segment(182.62890625,463.107421875,4.39013671875,7.025390625,-4.12890625,-6.607421875));
		obj3.add(new Segment(173.88671875,439.78125,-10.7880859375,5.611328125,23.61328125,-12.28125));
		obj3.add(new Segment(266.86376953125,437.4462890625,0,0,0,0));
		obj3.closed = true;
		//var obj4 = new Path(); // rectangle used in drawing to calc scale
		//obj4.add(new Segment(464.19384765625,435.857421875,0,0,0,0));  // width of pattern  = 389.992 pt
		//obj4.add(new Segment(74.2021484375,435.857421875,0,0,0,0));    
		//obj4.add(new Segment(74.2021484375,50.3291015625,0,0,0,0));    // height of pattern = 385.528 pt
		//obj4.add(new Segment(464.19384765625,50.3291015625,0,0,0,0));
		//obj4.closed = true;


        //----------------
		unit.appendTop( obj0);
		unit.appendTop( obj1);
		unit.appendTop( obj2);
		unit.appendTop( obj3);
		var scalex = userSize.x/obj0.bounds.width; // approximately same size
		var scaley = userSize.y/obj0.bounds.height;
		unit.scale( scalex, scaley );
		stepSize = new Point( 389.992*scalex, 385.528*scaley); // measured step from original drawing 
		unit.shear(shear/100,0);   
		rowsPerUnit = 2;
	}



  function defineRow () { // ----------------------------------

	  // repeat unit for steps at step distance
	  var xCount =  (targetType == rowcol) ? values.columns : targetSize.x / stepSize.x

	  for (var j=0; j < xCount; j++) {

		var newUnit = unit.clone(); 

		// handle x-flips 
		var objFlip = values.objFlip;
		if (objFlip == flip) {newUnit.scale(-1,1);  // invert all
		} else if (objFlip == flipAlt &&  Math.floor((j/2)) % 2 ) { newUnit.scale(1,-1);  // invert alts about y axis
		} else if (objFlip == flipAlt2 && Math.floor((j/4))% 2 ) {  newUnit.scale(1,-1);  // invert pairs about y axis
		}		
		newUnit.position = targetOrigin + new Point(stepSize.x/2 + j * stepSize.x, 0); 
		row.appendTop(newUnit);

	   }// loop to make row

   }//  fn defineRow

	function defineGroup() { // ----------------------------------
		// repeat row for rows at rowHeight distance
		// replicate the row ( easier to start from zero and discard the master row)
		
	     var yCount =  (targetType == rowcol) ? values.rows/rowsPerUnit : targetSize.y / stepSize.y
	     
	    var stagger = values.rowStagger
	    if(stagger) { yCount++} // stagger moves it to theleft so add one at the end


		for (var i=0; i < yCount; i++) {
		
			var newRow = row.clone()

			// handle row flips 
			var rowFlip = values.rowFlip;
			if (rowFlip == flip) {
				newRow.scale(-1,1);
			}else if (rowFlip == flipAlt &&  i % 2 ) {
				newRow.scale(1,-1);
			}else if (rowFlip == flipAlt2 && Math.floor((i/2))% 2 ) {
				newRow.scale(1,-1);  
			}
			
			if (stagger && i%2) {
			   newRow.translate(new Point(-stepSize.x/2, stepSize.y/2 + stepSize.y * i));; 
			}else{
				newRow.translate(new Point(0, stepSize.y/2 + stepSize.y * i));
			}
			// add to the overall group
			group.appendTop(newRow);

		}//loop rows into group

	}// fn defineGroup

    // main code of run routine ----------------------------
    
    defineTarget();
    
    defineUnit(); // basic unit, scaled to user request
    
    defineRow();
    
    defineGroup();
    
    // remove working items
    unit.remove();
    row.remove();
    
    

}// run


function rasterColour() { // ---------------------------------

    // use top raster and top path selection to colour the path
	var rasters = document.getItems({type: Raster,selected: true});
	var paths = document.getItems({type: Path, selected: true});

	if(rasters.length && paths.length){
		var raster = rasters[0];
		for(var i=0; i < paths.length; i++) {
			var path = paths[i];
			var colour = raster.getAverageColor(path);
			path.fillColor = colour;
		}
	}else{
		Dialog.alert('Please select a rasterized image and paths or groups of paths');
	}

}//fn rasterColour  



